----------------------------------------------------------------------------
-- spi_master.vhd
----------------------------------------------------------------------------
library IEEE; use IEEE.STD_LOGIC_1164.ALL;

entity spi_master is
 port (t: in  std_logic;
   start: in  std_logic;
   start_ack: buffer std_logic;         -- Ausgabe, die auch gelesen wird
   CS  : buffer std_logic := '1';       -- SPI Chip Select, Automatenzustand
   SDI : in     std_logic;              -- SPI serielle Eingabedaten
   SCK : buffer std_logic;              -- SPI Takt
   D: out std_logic_vector(7 downto 0));-- empfangenes Datenbyte
end entity;

architecture a of spi_master is
 signal idx: natural range 0 to 15 := 0;-- Bitzaehler 
 signal start_del: std_logic;           -- abgetastetes Startsignal
 signal ct: natural range 0 to 31;      -- Zaehler fuer Taktteiler
begin

 process(t) begin                       -- Erzeugung des Takts SCK mit der 
  if rising_edge(t) then
    if SCK = '0' then                     -- Frequenz 100MHz/50 = 2MHz
      if ct < 25 then ct <= ct +1;        
      else ct <= 0; SCK <= '1';
      end if;
    else
     if ct < 25 then ct <= ct +1;
     else ct <= 0; SCK <= '0';
     end if;
    end if;
  end if;
 end process;

 process(SCK)                          -- Empfangsprozess mit SPI-Takt
 begin
  if rising_edge(SCK) then              
   start_del <= start;                 -- Abtasten des Startsignals
   if start_del='0' then start_ack <= '0'; end if;
                                       -- Ruecknahme des Quittungssignals
   if CS='0' then                      -- Automatenzustand "CS aktiv"
    if idx>2 and idx < 11 then D(10 - idx) <= SDI; end if;
    if idx = 15 then CS <= '1'; idx <= 0;
	 else idx <= idx +1;                -- Bitzaehler erhoehen
    end if;
    
   else                                -- Automatenzustand "sonst"
    CS <= '1';                         -- CS inaktiv / Uebertragungspause 
	 if idx < 3 then idx <= idx +1;     -- Warte mind. 3 Bitzeiten
    elsif (start_del='1') and (start_ack = '0') then 
     CS <='0'; start_ack <='1'; idx <= 0;--Wechsel in Zustand "Uebertragung" 
    end if;
   end if;
  end if;
 end process;
end architecture;
